#! /usr/bin/env python

"""
    Bill McClain <wmcclain@salamander.com>
    November 14, 2004
    spamsplit.py

    The program recursively scans a directory tree, reads the contents of all
    mailboxes, and for each message that has 2 or more text attachments, displays
    a line giving:

        spamicity of the whole message
        spamcitiy of just the header
        spamicity of each text part

    I've only tested it with "mh" directories and Unix "mbx" files, but it
    should work with other formats supported by the Python library. The legal
    values for the --format= option are:

        mbx
        mmdf
        mh
        maildir
        babyl

    Access to the mailboxes is read-only, so experimentation should be
    harmless. But: USE AT YOUR OWN RISK. The Python library classes seem to
    silently skip non-mailbox files and directories (I handle maildir as a
    special case), so having other files in the directory tree should be ok.

    Example:

        spamsplit.py --format=mh ~/Mail/Spam

    History:
    
    v0.1
    Nov 14 2004
    Initial release. Note: the code does not currently decode base64 or
        quoted-printable parts, and so does not match bogofilter behavior
        in that respect. Only text/html and text/plain parts are considered.

"""

import sys, os.path, getopt, mailbox, email, os, cStringIO, re

mime_types = ("text/html",
              "text/plain")

# regular expression object to extract the spamicity value
prog = re.compile(r"spamicity=(?P<spamicity>.*),")

def usage():
    print "usage:"
    print "    spamsplit.py --format=[ mbx | mmdf | mh | maildir | babyl ] dir"
    print ""
    sys.exit()

def bogofilter(cmd, text):
    child_stdin, child_stdout = os.popen2(cmd)
    child_stdin.write(text)
    child_stdin.close()
    # should be only one return line
    for line in child_stdout:
        match = prog.search(line)
        return match.group("spamicity")
    
def score(message, parts):
    # score the whole message
    print "message", bogofilter("bogofilter -v", message.as_string(True)),
    # score the header
    buffer = cStringIO.StringIO()
    for header, value in message.items():
        print >> buffer, header + ": " + value
    print "header", bogofilter("bogofilter -v", buffer.getvalue()),
    # score each part
    for part in parts:
        print part.get_content_type(), 
        print bogofilter("bogofilter -v -H", part.as_string()),
        # would rather use this for decoding...but get an error:
        #print bogofilter("bogofilter -v -H", part.get_payload(0, True)),
    print

#
# This is for the directory-based mailboxes
#
def visitDirs(format, dirname, filenames):
    global maxlen
    if format == "mh":
        box = mailbox.MHMailbox(dirname, email.message_from_file)
    else:
        # Skip these maildir directories -- the mailbox.Maildir
        # module handles them.
        if os.path.basename(dirname) in ("cur","new","tmp"):
            return
        # We'll get an error if we encounter a non-maildir directory
        # in the tree. Just report the error and skip the directory.
        try:
            box = mailbox.Maildir(dirname, email.message_from_file)
        except Exception, value:
            print value
            return            
    message = box.next()
    while message:
        if message.is_multipart():
            parts = []
            for part in message.walk():
                if part.get_content_type() in mime_types:
                    if not part.is_multipart(): # shouldn't happen...but has
                        parts.append(part)
            if len(parts) >= 2:
                score(message, parts)
        message = box.next()

#
# This is for the file-based mailboxes
#
def visitFiles(format, dirname, filenames):
    global maxlen
    for fname in filenames:
        fullpath = os.path.join(dirname, fname)
        if not os.path.isfile(fullpath):
            continue
        if format == "mbx":
            box = mailbox.PortableUnixMailbox(file(fullpath), email.message_from_file)
        elif format == "mmdf":
            box = mailbox.MmdfMailbox(file(fullpath), email.message_from_file)
        else:
            box = mailbox.BabylMailbox(file(fullpath), email.message_from_file)
        message = box.next()
        while message:
            if message.is_multipart():
                parts = []
                for part in message.walk():
                    if part.get_content_type() in mime_types:
                        if not part.is_multipart(): # shouldn't happen...but has
                            parts.append(part)
                if len(parts) >= 2:
                    score(message, parts)
            message = box.next()

#
# Run the program!
#
# Get the command line arguments. We need a --format= value and a directory.
#
optlist, args = getopt.getopt(sys.argv[1:], "", ["format="])
if len(args) != 1:
    usage()
path = args[0]
if not os.path.isdir(path):
    print "Error: path must be a directory"
    usage()
if not optlist:
    usage()
for option, value in optlist:
    if option != "--format":
        usage()
    if value not in ("mbx", "mmdf", "mh", "maildir", "babyl"):
        usage()
    format = value

#
# Visit all subdirectories under "path".
#
if format in ("mh", "maildir"):
    os.path.walk(path, visitDirs, format)
else:
    os.path.walk(path, visitFiles, format)

    
